import React, { useRef, useEffect } from 'react';
import warning from "./utils/warning";
import Handler from "./Handler";
import { classList } from "./utils/util";

var noop = () => undefined;

var {
  stopPropagation
} = FocusEvent.prototype;

FocusEvent.prototype.stopPropagation = function () {
  this.path.forEach(node => {
    if (this.type === 'blur') {
      node.__onBlur && node.__onBlur();
    } else if (this.type === 'focus') {
      node.__onFocus && node.__onFocus();
    }
  });

  for (var _len = arguments.length, rest = new Array(_len), _key = 0; _key < _len; _key++) {
    rest[_key] = arguments[_key];
  }

  return stopPropagation.call(this, ...rest);
};
/***
 * 该组件的功能是类似于css伪类:focus-within
 * 即当前组件包裹的children的子组件获得焦点、那么当前组件包裹的children就会添加is-focus类
 */


var FocusWithin = props => {
  var {
    children,
    disabled,
    focusClassName = 'is-focus'
  } = props;
  var ref = useRef(null);
  useEffect(() => {
    var keys = Object.keys(props).filter(key => key !== 'children' && key !== 'disabled' && key !== 'focusClassName').map(key => key + '属性无效，');
    warning(keys.length === 0, 'FocusWithin', keys + 'FocusWithin组件只支持children、disabled、focusClassName属性');

    if (ref.current) {
      var __onFocus = ref.current.__onFocus || noop;

      ref.current.__onFocus = () => {
        handleFocus();
        return __onFocus();
      };

      var __onBlur = ref.current.__onBlur || noop;

      ref.current.__onBlur = () => {
        handleBlur();
        return __onBlur();
      };
    }
  }, []);
  /**
   * 处理聚焦
   */

  var handleFocus = () => {
    if (!disabled) {
      var classes = classList(ref.current);

      if (!classes.contains(focusClassName)) {
        classes.add(focusClassName);
      }
    }
  };
  /**
   * 处理失去焦点
   */


  var handleBlur = () => {
    if (!disabled) {
      var classes = classList(ref.current);

      if (classes.contains(focusClassName)) {
        classes.remove(focusClassName);
      }
    }
  };

  return /*#__PURE__*/React.createElement(Handler, {
    ref: ref,
    onFocus: handleFocus,
    onBlur: handleBlur
  }, children);
};

export default FocusWithin;