import _defineProperty from "@babel/runtime/helpers/defineProperty";

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

import React, { useRef, useEffect, useContext, forwardRef } from 'react';
import { once as onceFn } from "./utils/util";
import { findDOMNode } from 'react-dom';
import warning from "./utils/warning";
import EventTarget from "./utils/event";
import Handler from "./Handler";
var EVENT_ATTR = ['currentTarget', 'eventPhase', 'bubbles', 'cancelable', 'defaultPrevented', 'isTrusted', 'preventDefault', 'timeStamp', 'type', 'nativeEvent', 'target', 'altKey', 'button', 'buttons', 'clientX', 'clientY', 'ctrlKey', 'getModifierState', 'metaKey', 'movementX', 'movementY', 'pageX', 'pageY', 'relatedTarget', 'screenX', 'screenY', 'shiftKey', 'detail', 'view'];
var target = new EventTarget();
/**
 * document 事件处理函数
 */

function documentClickHandler(e) {
  target.dispatchEvent('click', e);
}

if (!window.__observerDocument__) {
  /**确保只监听一次**/

  /**由于react的合成事件也是监听的document 这里想要让documentClickHandler 必须先于react的合成事件之前就必须先于构造函数之前执行**/
  document.addEventListener('click', documentClickHandler, true);
  /**确保只监听一次**/

  window.__observerDocument__ = true;
}

var OutSideContext = /*#__PURE__*/React.createContext(undefined);
OutSideContext.displayName = 'OutSideContext';

var RefRenderFunction = function RefRenderFunction(props, forward) {
  var {
    children
  } = props;
  var {
    onFocus,
    onBlur,
    onOutSideClick,
    onClick,
    onMouseEnter,
    onMouseLeave,
    triggerTiming = 'inner',
    once = true
  } = props;

  if (typeof children !== 'object') {
    children = /*#__PURE__*/React.createElement("span", null, children);
    warning(false, 'OutSide', '传入的children不是JSX.Element类型的值');
  }

  var context = useContext(OutSideContext);
  var ins = useRef({});
  var ref = useRef(null);

  if (forward) {
    if (forward instanceof Function) {
      var originFn = forward;
      var current;

      ref = function ref(_ref) {
        originFn.bind(this)(_ref);
        current = _ref;
      };

      Object.defineProperty(ref, 'current', {
        get() {
          return findDOMNode(current);
        }

      });
    } else {
      ref = forward;
    }
  }
  /***
   * 组件加载后
   */


  useEffect(() => {
    if (onOutSideClick || onClick) {
      generateOnce();
      target.addEventListener('click', handleCallback);
    }

    return () => {
      target.removeEventListener('click', handleCallback);
    };
  }, []);
  /**
   * 生成一次执行函数
   */

  var generateOnce = () => {
    /**是否只能点击一次**/
    if (once) {
      ins.current.onceClick = onceFn(onOutSideClick);
    }
  };
  /**
   * 处理鼠标进入
   * @param e
   */


  var handleMouseEnter = e => {
    /**事件确保这里只需要在目标元素执行**/
    if (ref.current && ref.current.contains(e.target)) {
      ins.current.triggerInner = true;
      onMouseEnter && onMouseEnter(e);
    }
  };
  /**
   * 处理鼠标进入
   * @param e
   */


  var handleMouseLeave = e => {
    /**事件确保这里只需要在目标元素执行**/
    if (ref.current && ref.current.contains(e.target)) {
      onMouseLeave && onMouseLeave(e);
    }
  };
  /**
   * 生成event
   * @param e
   */


  var generateEvent = e => {
    if (!context) {
      return e;
    }

    var {
      stopPropagation
    } = context;
    var isPropagationStopped = false;
    var event = {
      stopPropagation() {
        if (e.stopPropagation) {
          e.stopPropagation();
        }

        stopPropagation && stopPropagation();
        isPropagationStopped = true;
      },

      isPropagationStopped() {
        return isPropagationStopped;
      }

    };
    EVENT_ATTR.forEach(attr => {
      Object.defineProperty(event, attr, {
        get() {
          /**对这个target属性做特殊处理**/
          if (attr === 'target') {
            return e.target || e.srcElement;
          }

          return e[attr];
        }

      });
    });
    return event;
  };
  /**
   * 处理组价的事件
   */


  var handleCallback = e => {
    var {
      current
    } = ref;

    if (current) {
      var _style = getComputedStyle(current);

      if (_style.pointerEvents === 'none') {
        warning(false, 'OutSide', '节点的样式 pointerEvents = none 导致onOutSideClick失效', current);
      }
    }
    /**ie中不存在e.target 只有e.srcElement**/


    if (!current.contains(e.target || e.srcElement)) {
      if (triggerTiming === 'outside' || triggerTiming === 'inner' && ins.current.triggerInner) {
        var callback;
        /**是否只执行一次**/

        if (once) {
          /**取出this.onceClick等函数**/
          callback = ins.current.onceClick;
          /**执行this.onceClick等函数**/

          callback && callback(e);
          /**表示当前是没有点击children**/

          ins.current.triggerInner = false;
        } else {
          /**如果不是执行一次那么就使用传下来的callback**/
          !ins.current.isPropagationStopped && onOutSideClick && onOutSideClick(generateEvent(e));
        }
      }
    }
  };
  /**
   *处理内部点击
   * @param e
   */


  var handleInnerClick = e => {
    ins.current.triggerInner = true;
    generateOnce();
    onClick && onClick(e);
  };

  var childProps = _objectSpread({}, children.props);

  return /*#__PURE__*/React.createElement(OutSideContext.Provider, {
    value: {
      stopPropagation: () => {
        ins.current.isPropagationStopped = true;
      }
    }
  }, /*#__PURE__*/React.createElement(Handler, {
    ref: ref,
    onClick: handleInnerClick,
    onFocus: onFocus,
    onBlur: onBlur,
    onMouseEnter: handleMouseEnter,
    onMouseLeave: handleMouseLeave
  }, /*#__PURE__*/React.cloneElement(children, childProps)));
};

var OutSide = /*#__PURE__*/forwardRef(RefRenderFunction);
OutSide.displayName = 'OutSide';
export default OutSide;