"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _warning = _interopRequireDefault(require("./utils/warning"));

var _Handler = _interopRequireDefault(require("./Handler"));

var _util = require("./utils/util");

var noop = function noop() {
  return undefined;
};

var stopPropagation = FocusEvent.prototype.stopPropagation;

FocusEvent.prototype.stopPropagation = function () {
  var _this = this;

  this.path.forEach(function (node) {
    if (_this.type === 'blur') {
      node.__onBlur && node.__onBlur();
    } else if (_this.type === 'focus') {
      node.__onFocus && node.__onFocus();
    }
  });

  for (var _len = arguments.length, rest = new Array(_len), _key = 0; _key < _len; _key++) {
    rest[_key] = arguments[_key];
  }

  return stopPropagation.call.apply(stopPropagation, [this].concat(rest));
};
/***
 * 该组件的功能是类似于css伪类:focus-within
 * 即当前组件包裹的children的子组件获得焦点、那么当前组件包裹的children就会添加is-focus类
 */


var FocusWithin = function FocusWithin(props) {
  var children = props.children,
      disabled = props.disabled,
      _props$focusClassName = props.focusClassName,
      focusClassName = _props$focusClassName === void 0 ? 'is-focus' : _props$focusClassName;
  var ref = (0, _react.useRef)(null);
  (0, _react.useEffect)(function () {
    var keys = Object.keys(props).filter(function (key) {
      return key !== 'children' && key !== 'disabled' && key !== 'focusClassName';
    }).map(function (key) {
      return key + '属性无效，';
    });
    (0, _warning.default)(keys.length === 0, 'FocusWithin', keys + 'FocusWithin组件只支持children、disabled、focusClassName属性');

    if (ref.current) {
      var __onFocus = ref.current.__onFocus || noop;

      ref.current.__onFocus = function () {
        handleFocus();
        return __onFocus();
      };

      var __onBlur = ref.current.__onBlur || noop;

      ref.current.__onBlur = function () {
        handleBlur();
        return __onBlur();
      };
    }
  }, []);
  /**
   * 处理聚焦
   */

  var handleFocus = function handleFocus() {
    if (!disabled) {
      var classes = (0, _util.classList)(ref.current);

      if (!classes.contains(focusClassName)) {
        classes.add(focusClassName);
      }
    }
  };
  /**
   * 处理失去焦点
   */


  var handleBlur = function handleBlur() {
    if (!disabled) {
      var classes = (0, _util.classList)(ref.current);

      if (classes.contains(focusClassName)) {
        classes.remove(focusClassName);
      }
    }
  };

  return /*#__PURE__*/_react.default.createElement(_Handler.default, {
    ref: ref,
    onFocus: handleFocus,
    onBlur: handleBlur
  }, children);
};

var _default = FocusWithin;
exports.default = _default;