"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _react = _interopRequireWildcard(require("react"));

var _util = require("./utils/util");

var _reactDom = require("react-dom");

var _warning = _interopRequireDefault(require("./utils/warning"));

var _event = _interopRequireDefault(require("./utils/event"));

var _Handler = _interopRequireDefault(require("./Handler"));

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

var EVENT_ATTR = ['currentTarget', 'eventPhase', 'bubbles', 'cancelable', 'defaultPrevented', 'isTrusted', 'preventDefault', 'timeStamp', 'type', 'nativeEvent', 'target', 'altKey', 'button', 'buttons', 'clientX', 'clientY', 'ctrlKey', 'getModifierState', 'metaKey', 'movementX', 'movementY', 'pageX', 'pageY', 'relatedTarget', 'screenX', 'screenY', 'shiftKey', 'detail', 'view'];
var target = new _event.default();
/**
 * document 事件处理函数
 */

function documentClickHandler(e) {
  target.dispatchEvent('click', e);
}

if (!window.__observerDocument__) {
  /**确保只监听一次**/

  /**由于react的合成事件也是监听的document 这里想要让documentClickHandler 必须先于react的合成事件之前就必须先于构造函数之前执行**/
  document.addEventListener('click', documentClickHandler, true);
  /**确保只监听一次**/

  window.__observerDocument__ = true;
}

var OutSideContext = /*#__PURE__*/_react.default.createContext(undefined);

OutSideContext.displayName = 'OutSideContext';

var RefRenderFunction = function RefRenderFunction(props, forward) {
  var children = props.children;
  var onFocus = props.onFocus,
      onBlur = props.onBlur,
      onOutSideClick = props.onOutSideClick,
      onClick = props.onClick,
      onMouseEnter = props.onMouseEnter,
      onMouseLeave = props.onMouseLeave,
      _props$triggerTiming = props.triggerTiming,
      triggerTiming = _props$triggerTiming === void 0 ? 'inner' : _props$triggerTiming,
      _props$once = props.once,
      once = _props$once === void 0 ? true : _props$once;

  if ((0, _typeof2.default)(children) !== 'object') {
    children = /*#__PURE__*/_react.default.createElement("span", null, children);
    (0, _warning.default)(false, 'OutSide', '传入的children不是JSX.Element类型的值');
  }

  var context = (0, _react.useContext)(OutSideContext);
  var ins = (0, _react.useRef)({});
  var ref = (0, _react.useRef)(null);

  if (forward) {
    if (forward instanceof Function) {
      var originFn = forward;
      var current;

      ref = function ref(_ref) {
        originFn.bind(this)(_ref);
        current = _ref;
      };

      Object.defineProperty(ref, 'current', {
        get: function get() {
          return (0, _reactDom.findDOMNode)(current);
        }
      });
    } else {
      ref = forward;
    }
  }
  /***
   * 组件加载后
   */


  (0, _react.useEffect)(function () {
    if (onOutSideClick || onClick) {
      generateOnce();
      target.addEventListener('click', handleCallback);
    }

    return function () {
      target.removeEventListener('click', handleCallback);
    };
  }, []);
  /**
   * 生成一次执行函数
   */

  var generateOnce = function generateOnce() {
    /**是否只能点击一次**/
    if (once) {
      ins.current.onceClick = (0, _util.once)(onOutSideClick);
    }
  };
  /**
   * 处理鼠标进入
   * @param e
   */


  var handleMouseEnter = function handleMouseEnter(e) {
    /**事件确保这里只需要在目标元素执行**/
    if (ref.current && ref.current.contains(e.target)) {
      ins.current.triggerInner = true;
      onMouseEnter && onMouseEnter(e);
    }
  };
  /**
   * 处理鼠标进入
   * @param e
   */


  var handleMouseLeave = function handleMouseLeave(e) {
    /**事件确保这里只需要在目标元素执行**/
    if (ref.current && ref.current.contains(e.target)) {
      onMouseLeave && onMouseLeave(e);
    }
  };
  /**
   * 生成event
   * @param e
   */


  var generateEvent = function generateEvent(e) {
    if (!context) {
      return e;
    }

    var _stopPropagation = context.stopPropagation;
    var _isPropagationStopped = false;
    var event = {
      stopPropagation: function stopPropagation() {
        if (e.stopPropagation) {
          e.stopPropagation();
        }

        _stopPropagation && _stopPropagation();
        _isPropagationStopped = true;
      },
      isPropagationStopped: function isPropagationStopped() {
        return _isPropagationStopped;
      }
    };
    EVENT_ATTR.forEach(function (attr) {
      Object.defineProperty(event, attr, {
        get: function get() {
          /**对这个target属性做特殊处理**/
          if (attr === 'target') {
            return e.target || e.srcElement;
          }

          return e[attr];
        }
      });
    });
    return event;
  };
  /**
   * 处理组价的事件
   */


  var handleCallback = function handleCallback(e) {
    var _ref2 = ref,
        current = _ref2.current;

    if (current) {
      var _style = getComputedStyle(current);

      if (_style.pointerEvents === 'none') {
        (0, _warning.default)(false, 'OutSide', '节点的样式 pointerEvents = none 导致onOutSideClick失效', current);
      }
    }
    /**ie中不存在e.target 只有e.srcElement**/


    if (!current.contains(e.target || e.srcElement)) {
      if (triggerTiming === 'outside' || triggerTiming === 'inner' && ins.current.triggerInner) {
        var callback;
        /**是否只执行一次**/

        if (once) {
          /**取出this.onceClick等函数**/
          callback = ins.current.onceClick;
          /**执行this.onceClick等函数**/

          callback && callback(e);
          /**表示当前是没有点击children**/

          ins.current.triggerInner = false;
        } else {
          /**如果不是执行一次那么就使用传下来的callback**/
          !ins.current.isPropagationStopped && onOutSideClick && onOutSideClick(generateEvent(e));
        }
      }
    }
  };
  /**
   *处理内部点击
   * @param e
   */


  var handleInnerClick = function handleInnerClick(e) {
    ins.current.triggerInner = true;
    generateOnce();
    onClick && onClick(e);
  };

  var childProps = _objectSpread({}, children.props);

  return /*#__PURE__*/_react.default.createElement(OutSideContext.Provider, {
    value: {
      stopPropagation: function stopPropagation() {
        ins.current.isPropagationStopped = true;
      }
    }
  }, /*#__PURE__*/_react.default.createElement(_Handler.default, {
    ref: ref,
    onClick: handleInnerClick,
    onFocus: onFocus,
    onBlur: onBlur,
    onMouseEnter: handleMouseEnter,
    onMouseLeave: handleMouseLeave
  }, /*#__PURE__*/_react.default.cloneElement(children, childProps)));
};

var OutSide = /*#__PURE__*/(0, _react.forwardRef)(RefRenderFunction);
OutSide.displayName = 'OutSide';
var _default = OutSide;
exports.default = _default;