"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.UUID = UUID;
exports.once = once;
exports.classList = classList;

/**
 * 随机生成UUID
 */
function UUID() {
  return 'uuid-xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
    var r = Math.random() * 16 | 0,
        v = c === 'x' ? r : r & 0x3 | 0x8;
    return v.toString(16);
  });
}
/**
 * 只执行一次fn
 * @param fn
 * @param context
 */


function once(fn, context) {
  var result;
  return function () {
    if (fn) {
      for (var _len = arguments.length, params = new Array(_len), _key = 0; _key < _len; _key++) {
        params[_key] = arguments[_key];
      }

      result = fn.apply(context || this, params);
      fn = null;
    }

    return result;
  };
}
/**
 * 类似于dom的classList
 */


function classList(dom) {
  var _dom$className;

  var _dom = dom;
  var classes = (dom === null || dom === void 0 ? void 0 : (_dom$className = dom.className) === null || _dom$className === void 0 ? void 0 : _dom$className.trim().split(/\s+/g)) || [];
  return {
    add: function add() {
      for (var _len2 = arguments.length, params = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        params[_key2] = arguments[_key2];
      }

      classes.push(params);
      classes = classes.flat();
      _dom && (_dom.className = classes.join(' ').trim());
    },
    remove: function remove() {
      for (var _len3 = arguments.length, params = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
        params[_key3] = arguments[_key3];
      }

      params.forEach(function () {
        var param = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
        var index = classes.indexOf(param);

        if (index !== -1) {
          classes.splice(index, 1);
        }
      });

      if (_dom && classes.length === 0) {
        return _dom.removeAttribute('class');
      }

      _dom && (_dom.className = classes.join(' ').trim());
    },
    toggle: function toggle(token, force) {
      var index = classes.indexOf(token);

      if (index === -1) {
        classes.push(token);
      } else if (!force) {
        classes.splice(index, 1);
      }

      if (_dom && classes.length === 0) {
        _dom.removeAttribute('class');

        return true;
      }

      _dom && (_dom.className = classes.join(' ').trim());
      return true;
    },
    contains: function contains(token) {
      return !!classes.find(function (item) {
        return token === item;
      });
    }
  };
}